import logging
import sys

import structlog

from .gnista_connetion import GnistaConnection, KeyringGnistaConnection, StaticTokenGnistaConnection
from .gnista_credential_manager import GnistaCredentialManager
from .gnista_data_point import GnistaDataPoint
from .gnista_data_points import GnistaDataPoints

logging.basicConfig(
    format="%(message)s",
    stream=sys.stdout,
    level=logging.INFO,
)


structlog.configure(
    processors=[
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.dev.ConsoleRenderer(),
    ],
    context_class=dict,
    logger_factory=structlog.stdlib.LoggerFactory(),
    wrapper_class=structlog.stdlib.BoundLogger,
    cache_logger_on_first_use=True,
)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.gnista_unit_response import GnistaUnitResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetConstantResponse")


@attr.s(auto_attribs=True)
class GetConstantResponse:
    """ """

    discriminator: str
    value: Union[Unset, float] = UNSET
    unit: Union[Unset, None, GnistaUnitResponse] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        value = self.value
        unit: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.unit, Unset):
            unit = self.unit.to_dict() if self.unit else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if value is not UNSET:
            field_dict["value"] = value
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        value = d.pop("value", UNSET)

        _unit = d.pop("unit", UNSET)
        unit: Union[Unset, None, GnistaUnitResponse]
        if _unit is None:
            unit = None
        elif isinstance(_unit, Unset):
            unit = UNSET
        else:
            unit = GnistaUnitResponse.from_dict(_unit)

        get_constant_response = cls(
            discriminator=discriminator,
            value=value,
            unit=unit,
        )

        get_constant_response.additional_properties = d
        return get_constant_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateConstantDataRequest")


@attr.s(auto_attribs=True)
class UpdateConstantDataRequest:
    """ """

    execution_id: Union[Unset, None, str] = UNSET
    value: Union[Unset, None, float] = UNSET
    unit: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        execution_id = self.execution_id
        value = self.value
        unit = self.unit

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if execution_id is not UNSET:
            field_dict["executionId"] = execution_id
        if value is not UNSET:
            field_dict["value"] = value
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        execution_id = d.pop("executionId", UNSET)

        value = d.pop("value", UNSET)

        unit = d.pop("unit", UNSET)

        update_constant_data_request = cls(
            execution_id=execution_id,
            value=value,
            unit=unit,
        )

        return update_constant_data_request



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.time_series_response import TimeSeriesResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetSeriesResponse")


@attr.s(auto_attribs=True)
class GetSeriesResponse:
    """ """

    discriminator: str
    curves: Union[Unset, None, List[TimeSeriesResponse]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        curves: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.curves, Unset):
            if self.curves is None:
                curves = None
            else:
                curves = []
                for curves_item_data in self.curves:
                    curves_item = curves_item_data.to_dict()

                    curves.append(curves_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if curves is not UNSET:
            field_dict["curves"] = curves

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        curves = []
        _curves = d.pop("curves", UNSET)
        for curves_item_data in _curves or []:
            curves_item = TimeSeriesResponse.from_dict(curves_item_data)

            curves.append(curves_item)

        get_series_response = cls(
            discriminator=discriminator,
            curves=curves,
        )

        get_series_response.additional_properties = d
        return get_series_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Generator, List, Optional

from structlog import get_logger

from data_point_client import AuthenticatedClient
from data_point_client.api.data_point import data_point_get_data_points_all
from data_point_client.models import DataPointResponseBase, EnDataPointExistenceDTO, ProblemDetails

from .gnista_connetion import GnistaConnection
from .gnista_data_point import GnistaDataPoint

log = get_logger()


class GnistaDataPoints:
    def __init__(self, connection: GnistaConnection):
        self.connection = connection

    def get_data_point_list(self) -> Generator[GnistaDataPoint, None, None]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(base_url=self.connection.datapoint_base_url, token=token)

        data_point_list = data_point_get_data_points_all.sync(client=client, existence=[EnDataPointExistenceDTO.FULL])
        if isinstance(data_point_list, ProblemDetails):
            problems: ProblemDetails = data_point_list
            raise Exception(problems)

        if isinstance(data_point_list, list):
            list_of_data_points: List[DataPointResponseBase] = data_point_list

        for data_point in list_of_data_points:
            name: Optional[str] = None
            if isinstance(data_point.name, str):
                name = data_point.name

            gnista_data_point = GnistaDataPoint(
                connection=self.connection, data_point_id=data_point.data_point_id, name=name
            )
            yield gnista_data_point



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="DataPointOrigin")


@attr.s(auto_attribs=True)
class DataPointOrigin:
    """ """

    discriminator: str

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        data_point_origin = cls(
            discriminator=discriminator,
        )

        return data_point_origin



import jwt
from oauth2_client.credentials_manager import CredentialManager
from structlog import get_logger

log = get_logger()


class GnistaCredentialManager(CredentialManager):
    def __init__(self, service_information, proxies=None):
        super().__init__(service_information, proxies)
        self.id_token = None
        self.access_token = None
        self.tenant_name = None
        self.company_name = None
        self.refresh_token = None

    def _process_token_response(self, token_response, refresh_token_mandatory):
        log.info("Token has been received.")

        access_token = token_response.get("access_token")

        id_token = token_response.get("id_token")
        if id_token is not None:
            decoded = jwt.decode(id_token, options={"verify_signature": False}, algorithms=["HS256", "RS256"])
            self.id_token = id_token
            self.tenant_name = decoded["tenant_name"]
            self.company_name = decoded["company_name"]

        super()._process_token_response(token_response, refresh_token_mandatory)
        self.access_token = access_token
        self.refresh_token = token_response.get("refresh_token")
        log.info("Successfully logged in", company=self.company_name, tenant_name=self.tenant_name)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="FileDTOResponse")


@attr.s(auto_attribs=True)
class FileDTOResponse:
    """ """

    file_id: str
    uploaded_at: datetime.datetime
    name: Union[Unset, None, str] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        file_id = self.file_id
        uploaded_at = self.uploaded_at.isoformat()

        name = self.name
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "fileId": file_id,
                "uploadedAt": uploaded_at,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_id = d.pop("fileId")

        uploaded_at = isoparse(d.pop("uploadedAt"))

        name = d.pop("name", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        file_dto_response = cls(
            file_id=file_id,
            uploaded_at=uploaded_at,
            name=name,
            tags=tags,
        )

        return file_dto_response



import json
import uuid
from typing import List, Union
from webbrowser import Error

import pandas as pd
from pandas import DataFrame
from structlog import get_logger

from data_point_client import AuthenticatedClient
from data_point_client.api.data_point import (
    data_point_get_data,
    data_point_get_data_2,
    data_point_get_data_point,
    data_point_get_data_point_2,
    data_point_update_constant_data,
    data_point_update_constant_data_2,
    data_point_update_time_series_data,
    data_point_update_time_series_data_2,
)
from data_point_client.models import GetConstantResponse, GetSeriesResponse
from data_point_client.models.get_data_request import GetDataRequest
from data_point_client.models.sub_series_request import SubSeriesRequest
from data_point_client.models.sub_series_request_values import SubSeriesRequestValues
from data_point_client.models.update_constant_data_request import UpdateConstantDataRequest
from data_point_client.models.update_time_series_request import UpdateTimeSeriesRequest
from data_point_client.types import Unset

from .gnista_connetion import GnistaConnection

log = get_logger()


class GnistaDataPoint:
    DATE_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
    DATE_NAME = "Date"
    VALUE_NAME = "Value"

    def __init__(self, connection: GnistaConnection, data_point_id: str, name: str = None):
        self.connection = connection
        self.data_point_id = data_point_id
        if name is None:
            self._load_details()
        else:
            self.name = name

    def __str__(self):
        return "GnistaDataPoint " + self.data_point_id + " with name " + self.name

    def _load_details(self):
        token = self.connection.get_access_token()
        client = AuthenticatedClient(base_url=self.connection.datapoint_base_url, token=token)
        if self.connection.id_token:
            data_point = data_point_get_data_point.sync(client=client, data_point_id=self.data_point_id)
        else:
            data_point = data_point_get_data_point_2.sync(
                client=client,
                data_point_id=self.data_point_id,
                tenant_name=self.connection.tenant_name,
            )
        if data_point is None:
            raise Error("Cannot load Datapoint")
        self.name = data_point.name

    # pylint: disable=too-many-arguments
    def get_data_point_data(
        self,
        request: GetDataRequest,
        post_fix: bool = False,
    ) -> Union[List[DataFrame], float, Unset]:
        token = self.connection.get_access_token()
        client = AuthenticatedClient(base_url=self.connection.datapoint_base_url, token=token)

        if self.connection.id_token:
            byte_content = data_point_get_data.sync_detailed(
                client=client, data_point_id=self.data_point_id, json_body=request
            ).content
        else:
            byte_content = data_point_get_data_2.sync_detailed(
                client=client,
                data_point_id=self.data_point_id,
                tenant_name=self.connection.tenant_name,
                json_body=request,
            ).content

        log.debug("Received Response from gnista.io", content=byte_content)

        content_text = byte_content.decode("utf-8")
        jscon_content = json.loads(content_text)

        series_response = GetSeriesResponse.from_dict(jscon_content)
        curves = series_response.curves

        if isinstance(curves, list):
            data_frames = []
            # pylint: disable=E1133
            for curve in curves:
                # pylint: enable=E1133
                data_frame = self._from_time_frames(time_frames=curve.to_dict(), post_fix=post_fix)
                data_frames.append(data_frame)

            return data_frames

        constant_response = GetConstantResponse.from_dict(jscon_content)
        if constant_response is not None:
            return constant_response.value

        return None

    def set_data_point_data(
        self, data: Union[List[DataFrame], float], unit: Union[str, None], execution_id: Union[str, None]
    ):
        token = self.connection.get_access_token()
        client = AuthenticatedClient(base_url=self.connection.datapoint_base_url, token=token)

        if isinstance(data, list):
            sub_series: List[SubSeriesRequest] = []
            for data_frame in data:
                data_dict = self._to_dict(data_frame)
                value = SubSeriesRequestValues.from_dict(src_dict=data_dict)
                request = SubSeriesRequest(values=value)
                sub_series.append(request)

            update_request = UpdateTimeSeriesRequest(
                sub_series=sub_series, time_series_id=str(uuid.uuid4()), unit=unit, execution_id=execution_id
            )

            if self.connection.id_token:
                data_point_update_time_series_data.sync_detailed(
                    client=client, data_point_id=self.data_point_id, json_body=update_request
                )
            else:
                data_point_update_time_series_data_2.sync_detailed(
                    tenant_name=self.connection.tenant_name,
                    client=client,
                    data_point_id=self.data_point_id,
                    json_body=update_request,
                )

        if isinstance(data, float):
            constant_request = UpdateConstantDataRequest(
                value=data,
                unit=unit,
                execution_id=execution_id,
            )
            if self.connection.id_token:
                data_point_update_constant_data.sync_detailed(
                    client=client,
                    data_point_id=self.data_point_id,
                    json_body=constant_request,
                )
            else:
                data_point_update_constant_data_2.sync_detailed(
                    tenant_name=self.connection.tenant_name,
                    client=client,
                    data_point_id=self.data_point_id,
                    json_body=constant_request,
                )

    def _to_dict(self, data_frame: DataFrame):
        result = {}
        dct = data_frame.to_dict()[self.VALUE_NAME]
        for key in dct.keys():
            value = dct[key]
            key_string = key.strftime(self.DATE_FORMAT)
            result[key_string] = value
        return result

    def _from_time_frames(self, time_frames: dict, post_fix: bool, date_format: str = DATE_FORMAT) -> DataFrame:

        if not isinstance(time_frames, dict):
            raise TypeError

        value_column_name = self.name

        if value_column_name is None:
            value_column_name = self.VALUE_NAME

        if post_fix:
            value_column_name = value_column_name + "_gnista.io_" + self.data_point_id

        log.debug("Reading data as Pandas DataFrame")

        data_record = []
        for date in time_frames:
            value = time_frames[date]
            data_record.append({self.DATE_NAME: date, value_column_name: value})

        data_frame = pd.DataFrame.from_records(data_record, columns=[self.DATE_NAME, value_column_name])

        data_frame[self.DATE_NAME] = pd.to_datetime(data_frame[self.DATE_NAME], format=date_format)

        data_frame[value_column_name] = pd.to_numeric(data_frame[value_column_name])

        data_frame = data_frame.set_index(data_frame[self.DATE_NAME])
        data_frame = data_frame.drop([self.DATE_NAME], axis=1)

        return data_frame



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.sub_series_request import SubSeriesRequest
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateTimeSeriesRequest")


@attr.s(auto_attribs=True)
class UpdateTimeSeriesRequest:
    """ """

    time_series_id: Union[Unset, str] = UNSET
    execution_id: Union[Unset, None, str] = UNSET
    sub_series: Union[Unset, None, List[SubSeriesRequest]] = UNSET
    warnings: Union[Unset, None, List[str]] = UNSET
    unit: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        time_series_id = self.time_series_id
        execution_id = self.execution_id
        sub_series: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sub_series, Unset):
            if self.sub_series is None:
                sub_series = None
            else:
                sub_series = []
                for sub_series_item_data in self.sub_series:
                    sub_series_item = sub_series_item_data.to_dict()

                    sub_series.append(sub_series_item)

        warnings: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.warnings, Unset):
            if self.warnings is None:
                warnings = None
            else:
                warnings = self.warnings

        unit = self.unit

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if time_series_id is not UNSET:
            field_dict["timeSeriesId"] = time_series_id
        if execution_id is not UNSET:
            field_dict["executionId"] = execution_id
        if sub_series is not UNSET:
            field_dict["subSeries"] = sub_series
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        time_series_id = d.pop("timeSeriesId", UNSET)

        execution_id = d.pop("executionId", UNSET)

        sub_series = []
        _sub_series = d.pop("subSeries", UNSET)
        for sub_series_item_data in _sub_series or []:
            sub_series_item = SubSeriesRequest.from_dict(sub_series_item_data)

            sub_series.append(sub_series_item)

        warnings = cast(List[str], d.pop("warnings", UNSET))

        unit = d.pop("unit", UNSET)

        update_time_series_request = cls(
            time_series_id=time_series_id,
            execution_id=execution_id,
            sub_series=sub_series,
            warnings=warnings,
            unit=unit,
        )

        return update_time_series_request



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.time_series_response_curve import TimeSeriesResponseCurve
from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeSeriesResponse")


@attr.s(auto_attribs=True)
class TimeSeriesResponse:
    """ """

    discriminator: str
    curve: Union[Unset, None, TimeSeriesResponseCurve] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        discriminator = self.discriminator
        curve: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.curve, Unset):
            curve = self.curve.to_dict() if self.curve else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "discriminator": discriminator,
            }
        )
        if curve is not UNSET:
            field_dict["curve"] = curve

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminator = d.pop("discriminator")

        _curve = d.pop("curve", UNSET)
        curve: Union[Unset, None, TimeSeriesResponseCurve]
        if _curve is None:
            curve = None
        elif isinstance(_curve, Unset):
            curve = UNSET
        else:
            curve = TimeSeriesResponseCurve.from_dict(_curve)

        time_series_response = cls(
            discriminator=discriminator,
            curve=curve,
        )

        time_series_response.additional_properties = d
        return time_series_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ConstantDataPointData")


@attr.s(auto_attribs=True)
class ConstantDataPointData:
    """ """

    status: Union[Unset, None, str] = UNSET
    error_message: Union[Unset, None, str] = UNSET
    unit: Union[Unset, None, str] = UNSET
    value: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status
        error_message = self.error_message
        unit = self.unit
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message
        if unit is not UNSET:
            field_dict["unit"] = unit
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = d.pop("status", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        unit = d.pop("unit", UNSET)

        value = d.pop("value", UNSET)

        constant_data_point_data = cls(
            status=status,
            error_message=error_message,
            unit=unit,
            value=value,
        )

        constant_data_point_data.additional_properties = d
        return constant_data_point_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.data_bucket_base import DataBucketBase
from ..models.data_point_origin import DataPointOrigin
from ..models.en_data_point_existence_dto import EnDataPointExistenceDTO
from ..models.en_data_point_state_dto import EnDataPointStateDTO
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataPointResponseBase")


@attr.s(auto_attribs=True)
class DataPointResponseBase:
    """ """

    data_point_id: str
    created_on: datetime.datetime
    name: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    created_by: Union[Unset, None, str] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    store: Union[Unset, None, DataBucketBase] = UNSET
    state: Union[Unset, EnDataPointStateDTO] = UNSET
    existence: Union[Unset, EnDataPointExistenceDTO] = UNSET
    error_details: Union[Unset, None, str] = UNSET
    origin: Union[Unset, None, DataPointOrigin] = UNSET
    used_in_calculations: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        data_point_id = self.data_point_id
        created_on = self.created_on.isoformat()

        name = self.name
        description = self.description
        created_by = self.created_by
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        store: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.store, Unset):
            store = self.store.to_dict() if self.store else None

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        existence: Union[Unset, str] = UNSET
        if not isinstance(self.existence, Unset):
            existence = self.existence.value

        error_details = self.error_details
        origin: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.origin, Unset):
            origin = self.origin.to_dict() if self.origin else None

        used_in_calculations: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.used_in_calculations, Unset):
            if self.used_in_calculations is None:
                used_in_calculations = None
            else:
                used_in_calculations = self.used_in_calculations

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dataPointId": data_point_id,
                "createdOn": created_on,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if tags is not UNSET:
            field_dict["tags"] = tags
        if store is not UNSET:
            field_dict["store"] = store
        if state is not UNSET:
            field_dict["state"] = state
        if existence is not UNSET:
            field_dict["existence"] = existence
        if error_details is not UNSET:
            field_dict["errorDetails"] = error_details
        if origin is not UNSET:
            field_dict["origin"] = origin
        if used_in_calculations is not UNSET:
            field_dict["usedInCalculations"] = used_in_calculations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        data_point_id = d.pop("dataPointId")

        created_on = isoparse(d.pop("createdOn"))

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        created_by = d.pop("createdBy", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        _store = d.pop("store", UNSET)
        store: Union[Unset, None, DataBucketBase]
        if _store is None:
            store = None
        elif isinstance(_store, Unset):
            store = UNSET
        else:
            store = DataBucketBase.from_dict(_store)

        _state = d.pop("state", UNSET)
        state: Union[Unset, EnDataPointStateDTO]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = EnDataPointStateDTO(_state)

        _existence = d.pop("existence", UNSET)
        existence: Union[Unset, EnDataPointExistenceDTO]
        if isinstance(_existence, Unset):
            existence = UNSET
        else:
            existence = EnDataPointExistenceDTO(_existence)

        error_details = d.pop("errorDetails", UNSET)

        _origin = d.pop("origin", UNSET)
        origin: Union[Unset, None, DataPointOrigin]
        if _origin is None:
            origin = None
        elif isinstance(_origin, Unset):
            origin = UNSET
        else:
            origin = DataPointOrigin.from_dict(_origin)

        used_in_calculations = cast(List[str], d.pop("usedInCalculations", UNSET))

        data_point_response_base = cls(
            data_point_id=data_point_id,
            created_on=created_on,
            name=name,
            description=description,
            created_by=created_by,
            tags=tags,
            store=store,
            state=state,
            existence=existence,
            error_details=error_details,
            origin=origin,
            used_in_calculations=used_in_calculations,
        )

        return data_point_response_base



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DataPointDataBase")


@attr.s(auto_attribs=True)
class DataPointDataBase:
    """ """

    status: Union[Unset, None, str] = UNSET
    error_message: Union[Unset, None, str] = UNSET
    unit: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        status = self.status
        error_message = self.error_message
        unit = self.unit

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = d.pop("status", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        unit = d.pop("unit", UNSET)

        data_point_data_base = cls(
            status=status,
            error_message=error_message,
            unit=unit,
        )

        return data_point_data_base



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.problem_details_extensions import ProblemDetailsExtensions
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProblemDetails")


@attr.s(auto_attribs=True)
class ProblemDetails:
    """ """

    type: Union[Unset, None, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    status: Union[Unset, None, int] = UNSET
    detail: Union[Unset, None, str] = UNSET
    instance: Union[Unset, None, str] = UNSET
    extensions: Union[Unset, None, ProblemDetailsExtensions] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        title = self.title
        status = self.status
        detail = self.detail
        instance = self.instance
        extensions: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.extensions, Unset):
            extensions = self.extensions.to_dict() if self.extensions else None

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop

        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if detail is not UNSET:
            field_dict["detail"] = detail
        if instance is not UNSET:
            field_dict["instance"] = instance
        if extensions is not UNSET:
            field_dict["extensions"] = extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        title = d.pop("title", UNSET)

        status = d.pop("status", UNSET)

        detail = d.pop("detail", UNSET)

        instance = d.pop("instance", UNSET)

        _extensions = d.pop("extensions", UNSET)
        extensions: Union[Unset, None, ProblemDetailsExtensions]
        if _extensions is None:
            extensions = None
        elif isinstance(_extensions, Unset):
            extensions = UNSET
        else:
            extensions = ProblemDetailsExtensions.from_dict(_extensions)

        problem_details = cls(
            type=type,
            title=title,
            status=status,
            detail=detail,
            instance=instance,
            extensions=extensions,
        )

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = prop_dict

            additional_properties[prop_name] = additional_property

        problem_details.additional_properties = additional_properties
        return problem_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import webbrowser
from typing import Optional

import keyring
from oauth2_client.credentials_manager import OAuthError, ServiceInformation
from structlog import get_logger

from .gnista_credential_manager import GnistaCredentialManager

log = get_logger()


# pylint: disable=too-many-instance-attributes
class GnistaConnection:
    scope = ["data-api"]

    # pylint: disable=too-many-arguments
    def __init__(
        self,
        base_url: Optional[str] = None,
        datapoint_base_url: Optional[str] = None,
        authentication_base_url: Optional[str] = None,
        client_id: str = None,
        client_secret: str = None,
        tenant_name: str = None,
    ):
        if base_url is None:
            base_url = "https://app.gnista.io"

        if datapoint_base_url is None:
            datapoint_base_url = base_url + "/datapoint"

        if authentication_base_url is None:
            authentication_base_url = base_url + "/authentication"

        self.base_url = base_url
        self.datapoint_base_url = datapoint_base_url
        self.authentication_base_url = authentication_base_url
        self.refresh_token: Optional[str] = None
        self.access_token: Optional[str] = None
        self.id_token: Optional[str] = None
        self.tenant_name: Optional[str] = tenant_name
        self.client_id = client_id
        self.client_secret = client_secret

    def __str__(self):
        token_available = self.access_token is not None
        return "Gnista Connection to" + self.base_url + " has token: " + token_available

    def _get_base_url(self) -> str:
        return self.base_url

    def get_access_token(self) -> str:
        if self.refresh_token is None:
            # pylint: disable=E1128
            self.refresh_token = self._load_refresh_token()

        if (self.access_token is None or self.id_token is None) and self.refresh_token is None:
            log.info("Starting First Time Login")
            if self.client_id is not None and self.client_secret is not None:
                log.info("Client Credentials Flow")
                self._create_tokens_client_credentials(
                    client_id=self.client_id, client_secret=self.client_secret, scope=self.scope
                )
            else:
                log.info("Auth Code Flow")
                self._create_tokens_code_flow()
        else:
            # refresh with existing refresh token
            if self.refresh_token is not None:
                try:
                    log.info("Using stored refresh Token to Login")
                    self._refresh_tokens(refresh_token=self.refresh_token)
                except OAuthError:
                    log.info("Error using refresh Token, try getting a new one", exc_info=True)
                    if self.client_id is not None and self.client_secret is not None:
                        log.info("Client Credentials Flow")
                        self._create_tokens_client_credentials(
                            client_id=self.client_id, client_secret=self.client_secret, scope=self.scope
                        )
                    else:
                        log.info("Auth Code Flow")
                        self._create_tokens_code_flow()

        if self.access_token is None:
            raise Exception("No Token available")
        return self.access_token

    def _get_service_info(
        self, scope: list = None, client_id: str = "python", client_secret: str = None
    ) -> ServiceInformation:
        if scope is None:
            scope = self.scope

        return ServiceInformation(
            self.authentication_base_url + "/connect/authorize",
            self.authentication_base_url + "/connect/token",
            client_id,
            client_secret,
            scope,
            False,
        )

    def _refresh_tokens(self, refresh_token: str):
        service_information = self._get_service_info()
        manager = GnistaCredentialManager(service_information)
        manager.init_with_token(refresh_token)

        self.tenant_name = manager.tenant_name
        self.access_token = manager.access_token
        self.refresh_token = manager.refresh_token
        self.id_token = manager.id_token

        if self.refresh_token is not None:
            self._store_refresh_token(refresh_token=self.refresh_token)

    def _create_tokens_code_flow(self, scope: list = None):
        if scope is None:
            scope = self.scope

        scope.append("openid")
        scope.append("profile")
        scope.append("offline_access")

        service_information = self._get_service_info(scope)

        manager = GnistaCredentialManager(service_information)
        # manager.init_with_client_credentials()
        redirect_uri = "http://localhost:4200/home"
        url = manager.init_authorize_code_process(redirect_uri=redirect_uri, state="myState")
        log.info("Authentication has been started. Please follow the link to authenticate with your user:", url=url)
        webbrowser.open(url)

        code = manager.wait_and_terminate_authorize_code_process()
        # From this point the http server is opened on 8080 port and wait to receive a single GET request
        # All you need to do is open the url and the process will go on
        # (as long you put the host part of your redirect uri in your host file)
        # when the server gets the request with the code (or error) in its query parameters

        manager.init_with_authorize_code(redirect_uri, code)
        # Here access and refresh token may be used with self.refresh_token
        self.tenant_name = manager.tenant_name
        self.access_token = manager.access_token
        self.refresh_token = manager.refresh_token
        self.id_token = manager.id_token

        if self.refresh_token is not None:
            self._store_refresh_token(refresh_token=self.refresh_token)

    def _create_tokens_client_credentials(self, client_id: str, client_secret: str, scope: list = None):
        if scope is None:
            scope = self.scope

        service_information = self._get_service_info(scope, client_id, client_secret)

        manager = GnistaCredentialManager(service_information)
        manager.init_with_client_credentials()

        # Here access and refresh token may be used with self.refresh_token
        self.access_token = manager.access_token

        if self.refresh_token is not None:
            self._store_refresh_token(refresh_token=self.refresh_token)

    def _store_refresh_token(self, refresh_token: str):
        pass

    # pylint: disable=R0201
    def _load_refresh_token(self) -> Optional[str]:
        return None


class StaticTokenGnistaConnection(GnistaConnection):
    def __init__(self, base_url: Optional[str] = None, refresh_token: Optional[str] = None):
        super().__init__(base_url=base_url)
        self.refresh_token = refresh_token

    def _store_refresh_token(self, refresh_token: str):
        pass

    def _load_refresh_token(self) -> Optional[str]:
        return self.refresh_token


class KeyringGnistaConnection(GnistaConnection):
    # pylint: disable=too-many-arguments
    def __init__(
        self,
        service_name: str = "gnista_library",
        base_url: Optional[str] = None,
        enable_store_refresh_token: bool = True,
        client_id: str = None,
        client_secret: str = None,
        tenant_name: str = None,
    ):
        super().__init__(base_url=base_url, client_id=client_id, client_secret=client_secret, tenant_name=tenant_name)
        self.enable_store_refresh_token = enable_store_refresh_token
        self.service_name = service_name

    def _get_token_name(self):
        return "__refresh_token__:" + super()._get_base_url()

    def clear_stored_token(self):
        keyring.delete_password(self.service_name, self._get_token_name())

    def _store_refresh_token(self, refresh_token: str):
        if self.enable_store_refresh_token:
            pass

        keyring.set_password(self.service_name, self._get_token_name(), refresh_token)

    def _load_refresh_token(self) -> Optional[str]:
        token = keyring.get_password(self.service_name, self._get_token_name())
        return token



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['data_point_client',
 'data_point_client.api',
 'data_point_client.api.data_export',
 'data_point_client.api.data_point',
 'data_point_client.api.files',
 'data_point_client.models',
 'gnista_library']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.1.0,<3.0.0',
 'attrs>=20.1.0,<22.0.0',
 'colorama>=0.4.4,<0.5.0',
 'httpx>=0.15.4,<0.19.0',
 'keyring>=23.1.0,<24.0.0',
 'oauth2-client>=1.2.1,<2.0.0',
 'pandas>=1.3.2,<2.0.0',
 'python-dateutil>=2.8.0,<3.0.0',
 'structlog>=21.1.0,<22.0.0']

setup_kwargs = {
    'name': 'gnista-library',
    'version': '1.2.0',
    'description': 'A client library for accessing gnista.io',
    'long_description': '[![Gitlab Pipeline](https://gitlab.com/campfiresolutions/public/gnista.io-python-library/badges/main/pipeline.svg)](https://gitlab.com/campfiresolutions/public/gnista.io-python-library/-/pipelines)  [![Python Version](https://img.shields.io/pypi/pyversions/gnista-library)](https://pypi.org/project/gnista-library/)  [![PyPI version](https://img.shields.io/pypi/v/gnista-library)](https://pypi.org/project/gnista-library/)  [![License](https://img.shields.io/pypi/l/gnista-library)](https://pypi.org/project/gnista-library/)  [![Downloads](https://img.shields.io/pypi/dm/gnista-library)](https://pypi.org/project/gnista-library/) \n\n# gnista-library\nA client library for accessing gnista.io\n\n## Tutorial\n### Create new Poetry Project\nNavigate to a folder where you want to create your project and type\n```shell\npoetry new my-gnista-client\ncd my-gnista-client\n```\n\n### Add reference to your Project\nNavigate to the newly created project and add the PyPI package\n```shell\npoetry add gnista-library\n``` \n\n### Your first DataPoint\nCreate a new file you want to use to receive data this demo.py\n\n```python\nfrom gnista_library import KeyringGnistaConnection, GnistaDataPoint, GnistaDataPoints\n\nconnection = KeyringGnistaConnection()\n\ndata_point_id = "56c5c6ff-3f7d-4532-8fbf-a3795f7b48b8"\ndata_point = GnistaDataPoint(connection=connection, data_point_id=data_point_id)\n\ndata_point_data = data_point.get_data_point_data()\n\nprint(data_point_data)\n```\n\nYou need to replace the `DataPointId` with an ID from your gnista.io workspace.\n\nFor example the DataPointId of this DataPoint `https://aws.gnista.io/secured/dashboard/datapoint/4684d681-8728-4f59-aeb0-ac3f3c573303` is `4684d681-8728-4f59-aeb0-ac3f3c573303`\n\n### Run and Login\nRun your file in poetry\'s virtual environment\n```console\n$ poetry run python demo.py\n2021-09-02 14:51.58 [info     ] Authentication has been started. Please follow the link to authenticate with your user: [gnista_library.gnista_connetion] url=https://aws.gnista.io/authentication/connect/authorize?client_id=python&redirect_uri=http%3A%2F%2Flocalhost%3A4200%2Fhome&response_type=code&scope=data-api%20openid%20profile%20offline_access&state=myState\n```\nIn order to login copy the `url` into your Browser and Login to gnista.io or, if allowed a browser window will open by itself.\n\n### Keystore\nOnce you loggedin, the library will try to store your access token in your private keystore. Next time you run your programm, it might request a password to access your keystore again to gain access to gnista.io\nPlease take a look at [Keyring](https://pypi.org/project/keyring/) for details.\n\n## Advanced Example\n### Show received Data in a plot\n```shell\npoetry new my-gnista-client\ncd my-gnista-client\npoetry add gnista-library\npoetry add structlib\npoetry add matplotlib\n```\n\n```python\nimport matplotlib.pyplot as plt\nfrom structlog import get_logger\n\nfrom gnista_library import KeyringGnistaConnection, GnistaDataPoint, GnistaDataPoints\n\nlog = get_logger()\n\nconnection = KeyringGnistaConnection()\n\ndata_point_id = "56c5c6ff-3f7d-4532-8fbf-a3795f7b48b8"\ndata_point = GnistaDataPoint(connection=connection, data_point_id=data_point_id)\n\ndata_point_data = data_point.get_data_point_data()\nlog.info("Data has been received. Plotting")\ndata_point_data.plot()\nplt.show()\n\n```\n\n### Filter by DataPoint Names\n```shell\npoetry new my-gnista-client\ncd my-gnista-client\npoetry add gnista-library\npoetry add structlib\npoetry add matplotlib\n```\n\n```python\nimport matplotlib.pyplot as plt\nfrom structlog import get_logger\n\nfrom gnista_library import KeyringGnistaConnection, GnistaDataPoint, GnistaDataPoints\n\nlog = get_logger()\n\nconnection = KeyringGnistaConnection()\n\ndataPoints = GnistaDataPoints(connection=connection)\ndata_point_list = list(dataPoints.get_data_point_list())\n\nfor data_point in data_point_list:\n    log.info(data_point)\n\n# Find Specific Data Points\nfiltered_data_points = filter(\n    lambda data_point: data_point.name.startswith("371880214002"), data_point_list\n)\nfor data_point in filtered_data_points:\n    # get the data\n    data_point_data = data_point.get_data_point_data()\n    log.info(data_point_data)\n    data_point_data.plot()\n    plt.show()\n\n```\n\n\n\n## Links\n**Website**\n[![gnista.io](https://www.gnista.io/assets/images/gnista-logo-small.svg)](gnista.io)\n\n**PyPi**\n[![PyPi](https://pypi.org/static/images/logo-small.95de8436.svg)](https://pypi.org/project/gnista-library/)\n\n**GIT Repository**\n[![Gitlab](https://about.gitlab.com/images/icons/logos/slp-logo.svg)](https://gitlab.com/campfiresolutions/public/gnista.io-python-library)',
    'author': 'Markus Hoffmann',
    'author_email': 'markus@campfire-solutions.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gnista.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.en_operator import EnOperator
from ..types import UNSET, Unset

T = TypeVar("T", bound="Rule")


@attr.s(auto_attribs=True)
class Rule:
    """ """

    number: Union[Unset, float] = UNSET
    op: Union[Unset, EnOperator] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        number = self.number
        op: Union[Unset, str] = UNSET
        if not isinstance(self.op, Unset):
            op = self.op.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if number is not UNSET:
            field_dict["number"] = number
        if op is not UNSET:
            field_dict["op"] = op

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        number = d.pop("number", UNSET)

        _op = d.pop("op", UNSET)
        op: Union[Unset, EnOperator]
        if isinstance(_op, Unset):
            op = UNSET
        else:
            op = EnOperator(_op)

        rule = cls(
            number=number,
            op=op,
        )

        return rule



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.en_data_point_status import EnDataPointStatus
from ..models.gnista_unit_response import GnistaUnitResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataPointDataResponse")


@attr.s(auto_attribs=True)
class DataPointDataResponse:
    """ """

    unit: Union[Unset, None, GnistaUnitResponse] = UNSET
    status: Union[Unset, EnDataPointStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        unit: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.unit, Unset):
            unit = self.unit.to_dict() if self.unit else None

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if unit is not UNSET:
            field_dict["unit"] = unit
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _unit = d.pop("unit", UNSET)
        unit: Union[Unset, None, GnistaUnitResponse]
        if _unit is None:
            unit = None
        elif isinstance(_unit, Unset):
            unit = UNSET
        else:
            unit = GnistaUnitResponse.from_dict(_unit)

        _status = d.pop("status", UNSET)
        status: Union[Unset, EnDataPointStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = EnDataPointStatus(_status)

        data_point_data_response = cls(
            unit=unit,
            status=status,
        )

        return data_point_data_response



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DataExportRequest")


@attr.s(auto_attribs=True)
class DataExportRequest:
    """ """

    data_point_id: str
    separator: str
    version: int
    unit: Union[Unset, None, str] = UNSET
    window_hours: Union[Unset, None, int] = UNSET
    file_name: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        data_point_id = self.data_point_id
        separator = self.separator
        version = self.version
        unit = self.unit
        window_hours = self.window_hours
        file_name = self.file_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dataPointId": data_point_id,
                "separator": separator,
                "version": version,
            }
        )
        if unit is not UNSET:
            field_dict["unit"] = unit
        if window_hours is not UNSET:
            field_dict["windowHours"] = window_hours
        if file_name is not UNSET:
            field_dict["fileName"] = file_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        data_point_id = d.pop("dataPointId")

        separator = d.pop("separator")

        version = d.pop("version")

        unit = d.pop("unit", UNSET)

        window_hours = d.pop("windowHours", UNSET)

        file_name = d.pop("fileName", UNSET)

        data_export_request = cls(
            data_point_id=data_point_id,
            separator=separator,
            version=version,
            unit=unit,
            window_hours=window_hours,
            file_name=file_name,
        )

        return data_export_request



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.time_series_period import TimeSeriesPeriod
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetDataRequest")


@attr.s(auto_attribs=True)
class GetDataRequest:
    """ """

    window_hours: Union[Unset, int] = UNSET
    version: Union[Unset, None, int] = UNSET
    time_series_periods: Union[Unset, None, List[TimeSeriesPeriod]] = UNSET
    demanded_unit: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        window_hours = self.window_hours
        version = self.version
        time_series_periods: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.time_series_periods, Unset):
            if self.time_series_periods is None:
                time_series_periods = None
            else:
                time_series_periods = []
                for time_series_periods_item_data in self.time_series_periods:
                    time_series_periods_item = time_series_periods_item_data.to_dict()

                    time_series_periods.append(time_series_periods_item)

        demanded_unit = self.demanded_unit

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if window_hours is not UNSET:
            field_dict["windowHours"] = window_hours
        if version is not UNSET:
            field_dict["version"] = version
        if time_series_periods is not UNSET:
            field_dict["timeSeriesPeriods"] = time_series_periods
        if demanded_unit is not UNSET:
            field_dict["demandedUnit"] = demanded_unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        window_hours = d.pop("windowHours", UNSET)

        version = d.pop("version", UNSET)

        time_series_periods = []
        _time_series_periods = d.pop("timeSeriesPeriods", UNSET)
        for time_series_periods_item_data in _time_series_periods or []:
            time_series_periods_item = TimeSeriesPeriod.from_dict(time_series_periods_item_data)

            time_series_periods.append(time_series_periods_item)

        demanded_unit = d.pop("demandedUnit", UNSET)

        get_data_request = cls(
            window_hours=window_hours,
            version=version,
            time_series_periods=time_series_periods,
            demanded_unit=demanded_unit,
        )

        return get_data_request
